/** ***************************************************************************
 * @file  File.cpp
 * @brief FileNX̎
 * @author Hidekazu Ito
 * @date 2015/05/09
 ******************************************************************************/

#define _CRT_SECURE_NO_WARNINGS

#include "File.h"
#include <tchar.h>
#ifdef USE_WINDOWS_SHELL
#include <shlobj.h>
#endif

using Win32Utility::File;

/** *****************************************************************
 * @brief RXgN^
 ********************************************************************/
File::File()
: m_hFind(INVALID_HANDLE_VALUE)
, m_FD()
{
    ZeroMemory(&m_FD, sizeof(m_FD));
#ifdef USE_WINDOWS_SHELL
    CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
#endif
}

/** *****************************************************************
 * @brief fXgN^
 ********************************************************************/
File::~File()
{
    if (m_hFind != INVALID_HANDLE_VALUE) {
        FindClose(m_hFind);
        m_hFind = INVALID_HANDLE_VALUE;
    }
#ifdef USE_WINDOWS_SHELL
    CoUninitialize();
#endif
}

/** *****************************************************************
 * @brief t@Ci̎擾j
 * @param[in]  filePath  t@C(pX)BChJ[h(*,?Ȃ)gp\
 * @return  Ԃ
 ********************************************************************/
DWORD File::SearchFile(LPCTSTR filePath)
{
    WIN32_FIND_DATA fd;
    DWORD findNum = 1;

    if (INVALID_HANDLE_VALUE != m_hFind) {
        FindClose( m_hFind );
    }
    // t@C
    m_hFind = FindFirstFile(filePath, &fd);
    if (INVALID_HANDLE_VALUE == m_hFind) {
        return 0;
    }
    // t@Č𐔂
    while (FALSE != FindNextFile(m_hFind, &fd)) {
        findNum++;
    }
    // nhŏɖ߂
    FindClose( m_hFind );
    m_hFind = FindFirstFile(filePath, &m_FD);

    return findNum;
}

/** *****************************************************************
 * @brief t@Cĩt@C擾j
 * @param[out]  fileName   t@C̊i[
 * @param[in]   maxLength  i[̃̕TCY(oCg)
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::GetNextSearchedFile(LPTSTR fileName, DWORD maxLength)
{
    if (INVALID_HANDLE_VALUE == m_hFind) {
        return FALSE;
    }
    _tcsncpy(fileName, m_FD.cFileName, maxLength);
    if (FALSE == FindNextFile(m_hFind, &m_FD)) {
        FindClose(m_hFind);
        m_hFind = INVALID_HANDLE_VALUE;
    }
    return TRUE;
}

/** *****************************************************************
 * @brief ut@CJv_CAO̕\
 * @param[out]  filePath   t@C(pX)̊i[
 * @param[in]   maxLength  i[敶̃TCY()
 * @param[in]   filter     tB^
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::GetOpenFile(LPTSTR filePath, DWORD maxLength, LPCTSTR filter)
{
    OPENFILENAME ofn = {};
    if (NULL == filePath) {
        return FALSE;
    }
    filePath[0] = '\0';
    ofn.lStructSize  = sizeof(OPENFILENAME);
    ofn.hwndOwner    = NULL;
    ofn.lpstrFilter  = filter;
    ofn.nFilterIndex = 1;
    ofn.lpstrFile    = filePath;
    ofn.nMaxFile     = maxLength;
    ofn.lpstrTitle   = TEXT("t@C̎擾");
    ofn.Flags        = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
    return ::GetOpenFileName(&ofn);
}

/** *****************************************************************
 * @brief uOtĕۑv_CAO̕\
 * @param[out]  filePath   t@C(pX)̊i[
 * @param[in]   maxLength  i[敶̃TCY()
 * @param[in]   filter     tB^
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::GetSaveFile(LPTSTR filePath, DWORD maxLength, LPCTSTR filter)
{
    OPENFILENAME ofn = {};
    if (NULL == filePath) {
        return FALSE;
    }
    filePath[0] = '\0';
    ofn.lStructSize  = sizeof(OPENFILENAME);
    ofn.hwndOwner    = NULL;
    ofn.lpstrFilter  = filter;
    ofn.nFilterIndex = 1;
    ofn.lpstrFile    = filePath;
    ofn.nMaxFile     = maxLength;
    ofn.lpstrTitle   = TEXT("Otĕۑ");
    ofn.Flags        = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
    return ::GetSaveFileName(&ofn);
}

/** *****************************************************************
 * @brief st@C̎擾
 * @param[out]  filePath   t@C(pX)̊i[
 * @param[in]   maxLength  i[敶̃TCY()
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::GetExeFile(LPTSTR filePath, DWORD maxLength)
{
    if (0 == GetModuleFileName(NULL, filePath, maxLength)) {
        return FALSE;
    } else {
        return TRUE;
    }
}

/** *****************************************************************
 * @brief stH_̎擾
 * @param[out]  filePath   tH_(pX)̊i[
 * @param[in]   maxLength  i[敶̃TCY()
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::GetExeFolder(LPTSTR filePath, DWORD maxLength)
{
    if (0 == GetModuleFileName(NULL, filePath, maxLength)) {
        return FALSE;
    } else {
        TCHAR szDir[_MAX_DIR];
        TCHAR szDrive[_MAX_DRIVE];
        TCHAR szFName[_MAX_FNAME];
        TCHAR szExt[_MAX_EXT];
        _tsplitpath(filePath, szDrive, szDir, szFName, szExt);
        _tcscpy(filePath, szDrive);
        _tcscat(filePath, szDir);
        return TRUE;
    }
}

#ifdef USE_WINDOWS_SHELL

/** *****************************************************************
 * @brief ItH_̎擾
 * @param[out]  filePath  tH_(pX)̊i[
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::GetFolder(TCHAR filePath[MAX_PATH])
{
    BROWSEINFO bi = {};
    LPITEMIDLIST idList;
    bi.hwndOwner      = NULL;
    bi.pidlRoot       = NULL;
    bi.pszDisplayName = NULL;
    bi.lpszTitle      = TEXT("擾tH_IĂ");
    bi.ulFlags        = BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT;
    bi.lpfn           = NULL;
    bi.lParam         = 0;
    bi.iImage         = 0;
    // tH__CAOJ
    idList = SHBrowseForFolder(&bi);
    if (NULL == idList) {
        return FALSE;
    }
    // ItH_̃tpX̎擾
    if (FALSE == SHGetPathFromIDList(idList, filePath)) {
        CoTaskMemFree(idList);
        return FALSE;
    }
    CoTaskMemFree(idList);
    return TRUE;
}

/** *****************************************************************
 * @brief t@C̎擾
 * @param[in]   filePath     t@C(pX)
 * @param[out]  typeName     t@C`̊̕i[
 * @param[out]  displayName  \t@C̊i[
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::GetFileInfo(LPCTSTR filePath, TCHAR typeName[80], TCHAR displayName[MAX_PATH])
{
    SHFILEINFO fi = {};
    if (0 == SHGetFileInfo(filePath, 0, &fi, sizeof(fi), SHGFI_TYPENAME | SHGFI_DISPLAYNAME)) {
        return FALSE;
    }
    if (NULL != typeName) {
        _tcscpy(typeName, fi.szTypeName);
    }
    if (NULL != displayName) {
        _tcscpy(displayName, fi.szDisplayName);
    }
    return TRUE;
}

/** *****************************************************************
 * @brief t@C̍폜
 * @param[in]  filePath      t@C(pX)
 * @param[in]  toRecycleBin  S~Ɉړ邩w(TRUE=S~)
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::DeleteFile(LPCTSTR filePath, BOOL toRecycleBin)
{
    if (FALSE == toRecycleBin) {
        return ::DeleteFile(filePath);
    } else {
        SHFILEOPSTRUCT fo = {};
        fo.hwnd              = NULL;
        fo.wFunc             = FO_DELETE;
        fo.pFrom             = filePath;
        fo.pTo               = NULL;
        fo.fFlags            = FOF_ALLOWUNDO | FOF_SIMPLEPROGRESS;
        fo.fAnyOperationsAborted = FALSE;
        fo.hNameMappings     = NULL;
        fo.lpszProgressTitle = NULL;
        if (0 != SHFileOperation(&fo)) {
            return FALSE;
        }
        return TRUE;
    }
}

/** *****************************************************************
 * @brief t@C̖OύX
 * @param[in]  fileName     t@C(pX)
 * @param[in]  newFileName  Vt@C(pX)
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::RenameFile(LPCTSTR fileName, LPCTSTR newFileName)
{
    SHFILEOPSTRUCT fo = {};
    fo.hwnd              = NULL;
    fo.wFunc             = FO_RENAME;
    fo.pFrom             = fileName;
    fo.pTo               = newFileName;
    fo.fFlags            = FOF_ALLOWUNDO | FOF_RENAMEONCOLLISION;
    fo.fAnyOperationsAborted = FALSE;
    fo.hNameMappings     = NULL;
    fo.lpszProgressTitle = NULL;
	int ret = SHFileOperation(&fo);
    if (0 != ret) {
        return FALSE;
    }
    return TRUE;
}

/** *****************************************************************
 * @brief t@C̃Rs[
 * @param[in]  filePath     Rs[̃t@C(pX)
 * @param[in]  newFilePath  Rs[̃t@C(pX)
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::CopyFile(LPCTSTR filePath, LPCTSTR newFilePath)
{
    SHFILEOPSTRUCT fo = {};
    fo.hwnd                 = NULL;
    fo.wFunc                = FO_COPY;
    fo.pFrom                = filePath;
    fo.pTo                  = newFilePath;
    fo.fFlags               = FOF_ALLOWUNDO;
    fo.fAnyOperationsAborted = FALSE;
    fo.hNameMappings        = NULL;
    fo.lpszProgressTitle    = NULL;
    if (0 != SHFileOperation(&fo)) {
        return FALSE;
    }
    return TRUE;
}

/** *****************************************************************
 * @brief t@C̈ړ
 * @param[in]  filePath     ړ̃t@C(pX)
 * @param[in]  newFilePath  ړ̃t@C(pX)
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL File::MoveFile(LPCTSTR filePath, LPCTSTR newFilePath)
{
    SHFILEOPSTRUCT fo = {};
    fo.hwnd                 = NULL;
    fo.wFunc                = FO_MOVE;
    fo.pFrom                = filePath;
    fo.pTo                  = newFilePath;
    fo.fFlags               = FOF_ALLOWUNDO;
    fo.hNameMappings        = NULL;
    fo.lpszProgressTitle    = NULL;
    if (0 != SHFileOperation(&fo)) {
        return FALSE;
    }
    return TRUE;
}

#endif

// EOF